<?php
# **********************************************************************************************
# phpLivePacker
#
# Copyright 2013 Christian Bartl (phplivepacker.bartlweb.net)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# This notice must appear in all copies of the script.


# browserdetection-lib.php
#
# detect browser and generate informations about it
# based on browser.php from Gary White
# *********************************************************************************************

 

class browserdetection {
	
	var $useragent;
	var $browser;
	var $version;
	var $major_version;
	var $engine;
	var $os;
	var $cacheConf;
	var $detectionConf;
	
	
/* ------------------------------------------------------------------------------------------------------------
   constructor

   @param useragent: custom useragent, if empty actual useragent form server environment is used
   @param cacheConf: phplivepacker caching configuration

------------------------------------------------------------------------------------------------------------ */

	function browserdetection($useragent, $conf) {
		if(isset($useragent)) {
			$this->useragent = $useragent;
		} else {
			$this->useragent = $_SERVER['HTTP_USER_AGENT'];
		}

		$this->cacheConf = $conf['caching'];
		$this->detectionConf = $conf['browserDetection'];
		
		// get cached information
		if($this->caching('read') == true) {
			return true;
		}
		
		// detect browser, version and engine
		$this->detectBrowser();
		
		// detect os
		$this->detectOS();
		
		// cache information
		$this->caching('write');
	}
	

/* ------------------------------------------------------------------------------------------------------------
   detect browser
------------------------------------------------------------------------------------------------------------ */

	function detectBrowser(){
		$this->browser = 'unknown';
		$this->version = 'unknown';
		$this->major_version = 'unknown';
		$this->engine = 'unknown';
		
		/* ----------------------------------------------------------
		   detect browsers
		------------------------------------------------------------- */

		// test for opera >= 15
		if (preg_match("/opr/i", $this->useragent) && !$this->detectionConf['detectOpera15asChrome']){
			$this->browser = "opera";
			
			$tmp=explode(" ", $this->useragent);
			$i = 0;
			foreach($tmp as $part) {
				if(preg_match("/Version/i", $part)) {
					$this->version = trim(substr($part,8));
				} else if(preg_match("/Opera/", $part)) {
					$this->version = str_replace("Opera/", "", $part);
				} else if($part == "Opera"){
					$this->version = $tmp[$i+1];
				}
				$i++;
			}		
		// test for opera < 15
		} else if (preg_match("/opera/i", $this->useragent)){
			$this->browser = "opera";
			
			$tmp=explode(" ", $this->useragent);
			$i = 0;
			foreach($tmp as $part) {
				if(preg_match("/Version/i", $part)) {
					$this->version = trim(substr($part,8));
				} else if(preg_match("/Opera/", $part)) {
					$this->version = str_replace("Opera/", "", $part);
				} else if($part == "Opera"){
					$this->version = $tmp[$i+1];
				}
				$i++;
			}
		// test for MS Internet Explorer version 1
		} else if (preg_match("/microsoft internet explorer/i", $this->useragent)){
			$this->browser = "ie";
			$this->version = "1.0";
			$var = stristr($this->useragent, "/");
			if (ereg("308|425|426|474|0b1", $var)){
				$this->version = "1.5";
			}

		// test for MS Internet Explorer
		} else if (preg_match("/msie/i",$this->useragent) && !preg_match("/opera/i", $this->useragent)){
			$val = explode(" ",stristr($this->useragent,"msie"));
			$this->browser = "ie";
			$this->version = str_replace(';', '', $val[1]);
		
		// test for Firefox
		} else if (preg_match("/Firefox/i", $this->useragent)){
			$this->browser="firefox";
			$val = stristr($this->useragent, "Firefox");
			$val = explode("/",$val);
			$this->version = $val[1];
			
	  	// test for Safari
		} else if (preg_match("/safari/i", $this->useragent) && !preg_match("/chrome/i", $this->useragent)){
			$this->browser = "safari";
			
			$tmp=explode(" ", $this->useragent);
			foreach($tmp as $part) {
				if (preg_match("/Version/i", $part)) {
					$this->version = trim(substr($part,8));
				}
			}			
		
		// test for Chrome
		} else if(preg_match("/chrome/i", $this->useragent)){
			$this->browser = "chrome";
			
			$tmp=explode(" ", $this->useragent);
			foreach($tmp as $part) {
				if (preg_match("/Chrome/i", $part)) {
					$this->version = trim(substr($part,7));
				}
			}
		}
		
		/* ----------------------------------------------------------
		   detect engines
		------------------------------------------------------------- */
		
		// test for webkit engine
		if(preg_match("/applewebkit/i", $this->useragent)){
			$this->engine = "webkit";
			
		// test for gecko engine
		} else if(preg_match("/gecko/i", $this->useragent)){
			$this->engine = "gecko";
		
		// test for trident engine
		} else if(preg_match("/trident/i", $this->useragent)){
			$this->engine = "trident";
		
		// test for presto engine
		} else if(preg_match("/presto/i", $this->useragent)){
			$this->engine = "presto";
		}
		
		/* ----------------------------------------------------------
		   generate major version
		------------------------------------------------------------- */
		
		$tmp = explode('.', $this->version);
		$this->major_version = $tmp[0];
	}


/* ------------------------------------------------------------------------------------------------------------
   detect operating system
------------------------------------------------------------------------------------------------------------ */

	function detectOS(){
		$this->os = 'unknown';
		
		if(preg_match("/windows ce/i", $this->useragent)) {
			$this->os = "wince";
		} else if(preg_match("/windows phone os/i", $this->useragent)) {
            $this->os = "windowsphone";
		} else if(preg_match("/win/i", $this->useragent)) {
			$this->os = "windows";
		} else if (preg_match("/iphone/i", $this->useragent) || preg_match("/ipod/i", $this->useragent) || preg_match("/ipad/i", $this->useragent)) {
			$this->os = "ios";
		} else if (preg_match("/mac/i", $this->useragent)) {
			$this->os = "osx";
		} else if (preg_match("/symbian/i", $this->useragent) || preg_match("/symbos/i", $this->useragent)) {
			$this->os = "symbian";
		} else if (preg_match("/android/i", $this->useragent)) {
			$this->os = "android";
        } else if (preg_match("/blackberry/i", $this->useragent)) {
            $this->os = "blackberry";		
		} else if (preg_match("/linux/i", $this->useragent)) {
			$this->os = "linux";
		}
	}

/* ------------------------------------------------------------------------------------------------------------
   caching
------------------------------------------------------------------------------------------------------------ */

	function caching($mode){
				
		if($this->cacheConf['caching'] == false || $this->cacheConf['cachingBrowserInfo'] == false) {
			return false;
		}
		
		$cacheFile = $this->cacheConf['cacheDirectory'] . 'browser_' . md5($this->useragent);
		
		if($mode == 'read') {
			if(file_exists($cacheFile)) {
				$content = file_get_contents($cacheFile);
				$this->setBrowserInformation(unserialize($content));
				
				return true;
			}
		} else if($mode == 'write') {
			$content = serialize($this->getBrowserInformation());

			$fp = fopen($cacheFile , 'w') or die();
			fwrite($fp , $content);
			fclose($fp);
		}
	}

/* ------------------------------------------------------------------------------------------------------------
   setter und getter
------------------------------------------------------------------------------------------------------------ */

	function getBrowserInformation(){
		$browserInfo = array(
			'useragent' => $this->useragent,
			'browser' => $this->browser,
			'version' => $this->version,
			'major_version' => $this->major_version,
			'engine' => $this->engine,
			'os' => $this->os
		);
		
		return $browserInfo;
	}
	
	function setBrowserInformation($data){		
		$this->useragent = $data['useragent'];
		$this->browser = $data['browser'];
		$this->version = $data['version'];
		$this->major_version = $data['major_version'];
		$this->engine = $data['engine'];
		$this->os = $data['os'];
	}
	

} // class browserdetection end

?>