<?php
# **********************************************************************************************
# phpLivePacker
#
# Copyright 2013 Christian Bartl (phplivepacker.bartlweb.net)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# This notice must appear in all copies of the script.


# phplivepacker-lib.php
#
# phplivepacker functions
# *********************************************************************************************

$cssImageArray = array();
 
 
class phplivepacker {
	
	var $baseDir;
	var $conf;
	var $setConf;
	var $type;
	var $set;
	var $browserInfo;
	var $content;
	
	
/* ------------------------------------------------------------------------------------------------------------
   constructor
------------------------------------------------------------------------------------------------------------ */

	function phplivepacker($configFile) {		
		// determine basedir
		$this->baseDir = str_replace('lib\phplivepacker-lib.php', '', realpath(__FILE__));
		$this->baseDir = str_replace('lib/phplivepacker-lib.php', '', $this->baseDir);
		
		// get configuration
		$this->getConf($this->baseDir . $configFile);
		
		// define content as array
		$this->content = array();
	}
	

/* ------------------------------------------------------------------------------------------------------------
   read config file
------------------------------------------------------------------------------------------------------------ */

	function getConf($configFile) {
		if(is_file($configFile)) {
			$this->conf = parse_ini_file($configFile, true);
			
			// process caching options
			if($this->conf['caching']['caching']) {                
                // check exclude by domain
                if(isset($this->conf['caching']['excludeDomains'])) {
	                $excludeDomains = explode(',', $this->conf['caching']['excludeDomains']);
	                if(sizeof($excludeDomains) > 0) {
	                    foreach($excludeDomains as $domain) {
	                        if(preg_match('/' . preg_quote(trim($domain)) . '$/', $_SERVER["SERVER_NAME"])) {
	                            $this->conf['caching']['caching'] = false;
								$this->conf['caching']['cachingBrowserInfo'] = false;
	                        }
	                    }
	                }
				}

                // check exclude by ip
                if(isset($this->conf['caching']['excludeIPs'])) {
	                $excludeIPs = explode(',', $this->conf['caching']['excludeIPs']);
	                if(sizeof($excludeIPs) > 0) {
	                    foreach($excludeIPs as $ip) {
	                        if($ip == $_SERVER["REMOTE_ADDR"]) {
	                        	$this->conf['caching']['caching'] = false;
								$this->conf['caching']['cachingBrowserInfo'] = false;
	                        }
	                    }
	                }
	        	}
            }
			
			// process and check cache directory
			$this->conf['caching']['cacheDirectory'] = $this->getRealPath($this->conf['caching']['cacheDirectory'], 'filesystem', false);

			if($this->conf['caching']['caching']) {
				if(!is_dir($this->conf['caching']['cacheDirectory'])) {
					print "/*\n" . 'ERROR: cache directory "' . $this->conf['caching']['cacheDirectory'] . '" not found."' . "\n*/";
					die();
				}
				
				if(!is_writeable($this->conf['caching']['cacheDirectory'])) {
					print "/*\n" . 'ERROR: cache directory "' . $this->conf['caching']['cacheDirectory'] . '" not writeable."' . "\n*/";
					die();
				}
			}
			
			// process development mode option
			if($this->conf['debug']['developmentMode']) {
					// always disable caching in development mode
				$this->conf['caching']['caching'] = false;
				
				if($this->conf['debug']['developmentModeDomains'] || $this->conf['debug']['developmentModeIPs']) {
					$this->conf['debug']['developmentMode'] = false;
				}

                // check activate by domain
                if(isset($this->conf['debug']['developmentModeDomains'])) {                	
	                $excludeDomains = explode(',', $this->conf['debug']['developmentModeDomains']);
	                if(sizeof($excludeDomains) > 0) {
	                    foreach($excludeDomains as $domain) {
	                        if(preg_match('/' . preg_quote(trim($domain)) . '$/', $_SERVER["SERVER_NAME"])) {
	                        	$this->conf['debug']['developmentMode'] = true;
	                        }
	                    }
	                }
				}

                // check activate by ip
                if(isset($this->conf['debug']['developmentModeIPs'])) {
	                $excludeIPs = explode(',', $this->conf['debug']['developmentModeIPs']);
	                if(sizeof($excludeIPs) > 0) {
	                    foreach($excludeIPs as $ip) {
	                        if($ip == $_SERVER["REMOTE_ADDR"]) {
	                        	$this->conf['debug']['developmentMode'] = true;
	                        }
	                    }
	                }
	        	}
            }
			
		} else {
			print 'ERROR: main configuration file "conf/phplivepacker.ini" could not be found.';
			die();
		}
	}

/* ------------------------------------------------------------------------------------------------------------
   process parameters
------------------------------------------------------------------------------------------------------------ */

	function processParameters() {		
		if(!isset($_GET['type'])) {
			print "/*\n" . 'ERROR: output type (css/js) not definied.' . "\n*/";
			die();
		}
		
		// determine type
		if($_GET['type'] != 'js' && $_GET['type'] != 'css') {
			print "/*\n" . 'ERROR: output type (css/js) not definied.' . "\n*/";
			die();
		}

		$this->type = 'css';
		if($_GET['type'] == 'js') {
			$this->type = 'js';
		}
		if($_GET['type'] == 'json') {
			$this->type = 'json';
		}

		// determine set
		$this->set = 'default';
		
		if(isset($_GET['set'])) {
			$this->set = $_GET['set'];
		}
	}

/* ------------------------------------------------------------------------------------------------------------
   get set configuration
------------------------------------------------------------------------------------------------------------ */

	function getSetConf() {		
		$setConfigFile = 'conf/sets/' . $this->set . '.ini';
		
		if(is_file($setConfigFile)) {
			$this->setConf = parse_ini_file($setConfigFile, true);
			
			if(!is_array($this->setConf)) {
				print "/*\n" . 'ERROR: set configuration file "' . $setConfigFile . '" for set "' . $this->set . '" is empty.' . "\n*/";
				die();
			} else {
				if(sizeof($this->setConf) == 0) {
					print "/*\n" . 'ERROR: set configuration file "' . $setConfigFile . '" for set "' . $this->set . '" is empty.' . "\n*/";
					die();
				}
			}
		} else {
			print "/*\n" . 'ERROR: set configuration file "' . $setConfigFile . '" for set "' . $this->set . '" could not be found.' . "\n*/";
			die();
		}
	}


/* ------------------------------------------------------------------------------------------------------------
   generate content
------------------------------------------------------------------------------------------------------------ */
	
	function generateContent(){
		foreach($this->setConf as $section => $files) {
			// process basic sections
			if($section == 'safe' || $section == 'base') {
				if(isset($files[$this->type])){
					if(is_array($files[$this->type])){
						foreach($files[$this->type] as $file) {
							$this->readFileContent($file, $section);
						}					
					}
				}
			
			// process browser and os dependent sections
			} else if($section != 'baseDir') {
				$allowedOperators = array('=', '!=', '<', '<=', '>', '>=');
				$matchConditionCount = 0;
    		
				// check conditions
				$conditions = explode(',', trim($section));

				foreach($conditions as $condition) {
					$condition = trim($condition);
					
					if($condition) {
						$termCount = 0;
						$matchTermCount = 0;
						$operator = '';
						
						// validate each single term in condition (AND operation)
						$terms = explode(' ', trim($condition));

						foreach($terms as $term) {
							$term = trim($term);
							
							if($term) {
								$termCount++;

								// detect operator
								if(in_array($term, $allowedOperators)) {
									$operator = $term;
								

								// test for operating system
								} else if(preg_match("/^\(.*\)$/", $term)) {
									$term = str_replace(array('(', ')'), '', $term);

									if(preg_match("/^!/", $term)) {
										$term = substr($term, '1');
										if($term != $this->browserInfo['os']) {
											$matchTermCount++;
										}
									} else {
										if($term == $this->browserInfo['os']) {
											$matchTermCount++;
										}
									}

								
								// test for browser version
								} else if(preg_match('/[0-9\.]/', $term)) {

									if(preg_match('/\./', $term)) {
										$version = $this->browserInfo['version'];
									} else {
										$version = $this->browserInfo['major_version'];
									}									
					
									if($operator) {
										if($operator == '=' || $operator == '==') {
											if($version == $term) {
												$matchTermCount = $matchTermCount + 2;
											}
										} else if($operator == '!=') {
											if($version != $term) {
												$matchTermCount = $matchTermCount + 2;
											}
										} else if($operator == '<') {
											if($version < $term) {
												$matchTermCount = $matchTermCount + 2;
											}
										} else if($operator == '<=') {
											if($version <= $term) {
												$matchTermCount = $matchTermCount + 2;
											}
										} else if($operator == '>') {
											if($version > $term) {
												$matchTermCount = $matchTermCount + 2;
											}
										} else if($operator == '>=') {
											if($version >= $term) {
												$matchTermCount = $matchTermCount + 2;
											}
										}
									} else {									
										if($version == $term) {
											$matchTermCount++;
										}
									}
									
								 
								// test for browser and engine
								} else {
									if(preg_match("/^!/", $term)) {
										$term = substr($term, '1');
										if($term != $this->browserInfo['browser'] && $term != $this->browserInfo['engine']) {
											$matchTermCount++;
										}
									} else {
										if($term == $this->browserInfo['browser'] || $term == $this->browserInfo['engine']) {
											$matchTermCount++;
										}	
									}													
								}
							}
						}
						
						// check if or condition matches
						if($matchTermCount == $termCount) {
							$matchConditionCount++;
						}
					}
				}

				// read files if condition matches
				if($matchConditionCount > 0){
					if(isset($files[$this->type])){							
						if(is_array($files[$this->type])){
							foreach($files[$this->type] as $file) {
								$this->readFileContent($file, $section);
							}					
						}
					}
				}    					
			}
		}
	}
	

/* ------------------------------------------------------------------------------------------------------------
   read file content
   
   @param file: file path given in set configuration
   @param section: section name from set configuration
------------------------------------------------------------------------------------------------------------ */	
	
	function readFileContent($file, $section){
		// create relative path
		$basedir = trim($this->setConf['baseDir'][$this->type], '/\\');
		$file = trim($file, '/\\');		
		$relpath = $basedir . '/' . $file;
		
		$realfile = $this->getRealPath($relpath, 'filesystem', true);

		if(file_exists($realfile)) {
			// generate css output
			if($this->type == 'css') {

				// debug output
				if($this->conf['debug']['developmentMode']) {
					$this->content[] = "@import url('" . $this->getRealPath($relpath, 'cssdev', true) . "');" . "\n";					

				// regular output
				} else {
					// read file
					$content = file_get_contents($realfile);
					
					// clear css
					$content = $this->clearCSS($content, $relpath);
					
					// minify css
					if($section != 'safe') {
						$content = $this->minifyContent($content);
					}
					
					// return to output
					$this->content[] = $content;
				}
				
			// generate js output
			} else if($this->type == 'js') {
				
				// read file
				$content = file_get_contents($realfile);

				// minify js
				if($section != 'safe') {
					if($this->conf['debug']['developmentMode']) {
						if($this->conf['debug']['developmentModeMinify']) {
							$content = $this->minifyContent($content);
						}
					} else {
						$content = $this->minifyContent($content);
					}
				}
					
				// add file header in development mode and do not minify
				if($this->conf['debug']['developmentMode']) {
				 	$debug =  "/* ***********************************************************************************************************************************************************************" . "\n";
                    $debug .= " * ***********************************************************************************************************************************************************************" . "\n";
					$debug .= " * " . "\n";
                    $debug .= " * " . $file . ' (' . $relpath . ')' . "\n";
					$debug .= " * " . "\n";
                    $debug .= " * ***********************************************************************************************************************************************************************" . "\n";
                    $debug .= " * *********************************************************************************************************************************************************************** */" . "\n" ;
					
					$content = $debug . "\n\n" . $content . "\n\n";
				}
				
				// return to output
				$this->content[] = $content;
				
			}

		} else {
			if($this->conf['debug']['debugMode']) {
				print("\n/*\n");
				print('file "' . $file . '" from section "' . $section . '" not found');
				print("\n*/\n");
				ob_flush();
			}
		}
	}


/* ------------------------------------------------------------------------------------------------------------
   clear css files
   
   @param content: css code
   @return cleared css code
------------------------------------------------------------------------------------------------------------ */	
	
	function clearCSS($content, $file){
		// remove @charset, because it is only allowed at the beginning of a file
		$content = preg_replace('/@charset\s".*?";/s', '', $content);
			
		// include images as base64 encoded string and/or correct image urls
		$path = $this->getRealPath($file, 'url', true);
		$imageHandlingConf = $this->getConfig('imageHandling');
		$content = preg_replace_callback(
			'/url\s*\(\s*["\\\']?\s*(?!http:\/\/|https:\/\/|\/|data:)([^"\\\'\)]*)\s*["\\\']?\s*\)/',
			function($string) use ($path, $imageHandlingConf) {
				return phplivepacker::processCSSImages($string[1], $path, $imageHandlingConf);
			},
			$content);

		// return content
		return $content;
	}


/* ------------------------------------------------------------------------------------------------------------
   process css images
   
   @param string: original css file path
   @param path: base path of actual css file
   @param conf: imageHandling configuration
   @return image string
------------------------------------------------------------------------------------------------------------ */	
	
	function processCSSImages($string, $path, $conf){
		global $cssImageArray;
		
		// rewrite image path
		$image = $path . $string;
		
		// include image as base64 encoded string
		if($conf['includeImages']) {
			// determine correct path
			$imagepath = realpath($_SERVER['DOCUMENT_ROOT'] . $image);
			if(!$imagepath) {
				$imagepath = realpath($path . $string);
			}

			// process image			
			$filetypes = array_map('trim', explode(',', $conf['allowedFileTypes']));
	        if(file_exists($imagepath)) {
	        	$filetype = pathinfo($imagepath, PATHINFO_EXTENSION);
				if (in_array($filetype, $filetypes)){
					if(filesize($imagepath) <= (intval($conf['filesizeLimit'])*1024)) {
						$imgbinary = fread(fopen($imagepath, "r"), filesize($imagepath));
						$image = 'data:image/' . $filetype . ';base64,' . base64_encode($imgbinary);
					} else {
						 $image = $path . $string;
						 $cssImageArray[] = $image;
					}
		        } else {
		            $image = $path . $string;
					$cssImageArray[] = $image;
		        }		        
	        } else {
	        	$image = $path . $string;
				$cssImageArray[] = $image;
	        }
		} else{
			$cssImageArray[] = $image;
		}	

		// return content
		return 'url(' . $image . ')';
	}

/* ------------------------------------------------------------------------------------------------------------
   minify content
   
   @param content: css or js code
   @return minified css code
------------------------------------------------------------------------------------------------------------ */	

	function minifyContent($content){
		// check if minifiying is enabled
		if(!$this->conf['output']['minify_' . $this->type]) {
			return $content;
		}		
			
		//remove comments
		if($this->type == 'css') {
			// remove single and multi line /* */ comments
			$content = preg_replace('#(?:/\*(?:[^*]|(?:\*+[^*/]))*\*+/)#', '', $content);
			
			// remove single line // comments
			$content = explode("\n", $content);
			foreach($content as $key => $val) {
				if(!preg_match('#data:.*;base64,#', $val)){
					$content[$key] = preg_replace('#(?<!http:|https:|ftp:|url\(|url\("|url\(\\\')//.*#', '', $val);
				}
			}
			$content = implode("\n", $content);
		} else if($this->type == 'js') {
			// remove single and multi line /* */ comments
			$content = preg_replace('#(?:/\*(?:[^*]|(?:\*+[^*/]))*\*+/)#', '', $content);
			
			// remove single line // comments
			$content = explode("\n", $content);
			foreach($content as $key => $val) {
				$content[$key] = preg_replace('#(^\s*|[;,{}}\[\]\(\)]\s*)(//.*$)#', '$1', $val);
			}
			$content = implode("\n", $content);
		}
			
		
		// remove extra spaces and line breaks
		$content = str_replace(array("\r\n", "\r", "\n", "\t", '  ', '    ', '    '), '', $content);
		$content = str_replace('{ ', '{', $content);
	    $content = str_replace(' }', '}', $content);
	    $content = str_replace('; ', ';', $content);
	    $content = str_replace(', ', ',', $content);
	    $content = str_replace(' {', '{', $content);
	    $content = str_replace('} ', '}', $content);
	    $content = str_replace(': ', ':', $content);
	    $content = str_replace(' ,', ',', $content);
	    $content = str_replace(' ;', ';', $content);
		
		// return content
		return $content;
	}


/* ------------------------------------------------------------------------------------------------------------
   return content
------------------------------------------------------------------------------------------------------------ */
	
	function returnContent(){
		// print content array
		print(implode($this->content, "\n"));
		
		// flush content cache
		ob_flush();
	}
	
	
/* ------------------------------------------------------------------------------------------------------------
   get cached content
------------------------------------------------------------------------------------------------------------ */
	
	function getCache(){
		if($this->conf['caching']['caching']) {
			$filename = $this->getCacheFileName();
			
			if(file_exists($filename)) {
				// set caching headers
				$filedate =  filemtime($filename);									
				$this->setHttpHeaderCaching($filedate);
				
				// return file content
				readfile($filename);
				exit();
			}
		}
	}

/* ------------------------------------------------------------------------------------------------------------
   write content to cache
------------------------------------------------------------------------------------------------------------ */
	
	function writeCache(){
		if($this->content) {
						
			// write file cache
			if($this->conf['caching']['caching']) {
				$filename = $this->getCacheFileName();
				
				$fp = fopen($filename , 'w') or die();
				fwrite($fp , implode($this->content, "\n"));
				fclose($fp);
			}
			
			// write css image array
			if($this->type == 'css' && $this->conf['imageHandling']['generateImageArray']) {
				$filename = $this->getCacheFileNameImageArray();
				
				$fp = fopen($filename , 'w') or die();
				fwrite($fp , $this->generateImageArray());
				fclose($fp);
			}
		}
	}
	

/* ------------------------------------------------------------------------------------------------------------
   generate image array
------------------------------------------------------------------------------------------------------------ */

	function generateImageArray() {
 		global $cssImageArray;
		
		// distinct
		$cssImageArray = array_unique($cssImageArray);
		
		// generate file
		if($this->conf['imageHandling']['imageArrayType'] == 'json') {
			$tmp = array($this->conf['imageHandling']['imageArrayPrefix'] . $this->set => $cssImageArray);
			$content = json_encode($tmp); 
		} else {
			$content = $this->conf['imageHandling']['imageArrayPrefix'] . $this->set . ' = new Array(';
			foreach($cssImageArray as $val) {
				$tmp[] = "'" . $val . "'";
			}
			$content .= implode(',', $tmp);
			$content .= ');';
		}
		
		return $content;
 	}

/* ------------------------------------------------------------------------------------------------------------
   debug function

   @param mode: determines debug output - conf/browserinfo/output/getcache/writecache
------------------------------------------------------------------------------------------------------------ */
	
	function debug($mode){		
		// do not return output if debug mode is disabled
		if(!$this->conf['debug']['debugMode']) {
			return false;
		}
		
		// generate output
		$output = array();
		
		$output[] = '';	
		$output[] = '/*';
		
		if($mode == 'conf') {
			$output[] = '====================================================================';
			$output[] = 'params';
			$output[] = '====================================================================';
			
			$output[] = 'type: ' . $this->type;
			$output[] = 'set: ' . $this->set;
			
			$output[] = '';			
			$output[] = '====================================================================';
			$output[] = 'config';
			$output[] = '====================================================================';
			
			$output[] = print_r($this->conf, true);
			
			$output[] = '====================================================================';
			$output[] = 'set config';
			$output[] = '====================================================================';
			
			$output[] = print_r($this->setConf, true);
						
			$output[] = '====================================================================';
			
		} else if($mode == 'browserinfo') {
			$output[] = '====================================================================';
			$output[] = 'browser information';
			$output[] = '====================================================================';
			
			if($this->conf['browserDetection']['overrideDetection']){
				$output[] = 'overrideDetection enabled';
			} else {
				$output[] = 'overrideDetection disabled';
			}
			$output[] = '';
			
			$output[] = print_r($this->browserInfo, true);
			
			$output[] = '====================================================================';
			
		} else if($mode == 'output') {
			$output[] = '====================================================================';
			$output[] = 'output options';
			$output[] = '====================================================================';
			
			$output[] = $this->conf['output']['fileheader_' . $this->type];
			
			if($this->conf['output']['gzip']){
				$output[] = 'gzip enabled';
			} else {
				$output[] = 'gzip disabled';
			}
			
			$output[] = '';
			$output[] = '====================================================================';
		} else if($mode == 'getcache') {
			$output[] = '====================================================================';
			$output[] = 'get cache';
			$output[] = '====================================================================';
			
			if($this->conf['caching']['caching']){
				$output[] = 'caching enabled';
			} else {
				$output[] = 'caching disabled';
			}
			
			$output[] = 'cache file: ' . $this->getCacheFileName();
			
			if(is_file($this->getCacheFileName())){
				$output[] = 'cached content found';
			} else {
				$output[] = 'no cached content found';
			}
			
			$output[] = '';
			$output[] = '====================================================================';
		} else if($mode == 'writecache') {
			$output[] = '====================================================================';
			$output[] = 'write cache';
			$output[] = '====================================================================';
			
			if($this->conf['caching']['caching']){
				$output[] = 'caching enabled';
			} else {
				$output[] = 'caching disabled';
			}	
			
			$output[] = 'cache file: ' . $this->getCacheFileName();
			
			if($this->content){
				$output[] = 'content generated';
			} else {
				$output[] = 'no content found';
			}
			
			if(is_file($this->getCacheFileName())){
				$output[] = 'cache file written';
			} else {
				$output[] = 'no cache file written';
			}
			
			$output[] = '';
			$output[] = '====================================================================';
		}
		
		$output[] = '*/';
		$output[] = '';
		
		// return output
		print(implode($output, "\n"));
		ob_flash;
	}


/* ------------------------------------------------------------------------------------------------------------
   get real path
  
   @param path: relative path from phplivepacker basedir
   @param mode: filesystem/url/webpath/cssdev
   @param isFile: true/false
   @return: full filesystem path, absolute url path from document root, full web path from document root or single processing path for css files in dev mode
------------------------------------------------------------------------------------------------------------ */

	public function getRealPath($path, $mode, $isFile){		
		// determine os
		$windows = false;
		
		if(isset($_ENV['OS'])) {
			if(preg_match("/^Windows/", $_ENV['OS'])) {
				$windows = true;
			}
		}
		if(isset($_SERVER['WINDIR'])) {
			$windows = true;
		}
		
		// set and repair slashes
		if($windows == true) {
			$slash = '\\';
			$path = str_replace('/', '\\', $path);
		} else {
			$slash = '/';
			$path = str_replace('\\', '/', $path);
		}
		
		// add trailing slash if it is no file
		if(!$isFile) {
			$path = rtrim($path, $slash);
			$path = $path . $slash;
		}
		
		// return absolute path from document root if mode is url, else return full filesystem path
		if($mode == 'url') {			
			$path = preg_replace('/\/[^\/]*$/', '/', $_SERVER['REQUEST_URI']) . str_replace('\\', '/', $path);
			$path = preg_replace('/[^\/]*$/', '', $path);
		} else if ($mode == 'webpath') {
			$path = str_replace('\\', '/', $path);
		} else if ($mode == 'cssdev') {
			$path = str_replace('\\', '/', $path);
			$path = str_replace('phplivepacker.php', 'phplivepacker-dev.php', $_SERVER["SCRIPT_NAME"]) . '?type=css&file=' . rawurlencode($path);
		}  else {
			$path = $this->baseDir . $path;
		}

		return $path;
	}


/* ------------------------------------------------------------------------------------------------------------
   helpers
------------------------------------------------------------------------------------------------------------ */

	function getCacheFileName(){
		$filename = $this->conf['caching']['cacheDirectory'];
		
		// type
		$filename .= $this->type;
		
		// set
		$filename .= '_' . $this->set;
		
		// browser
		if(is_array($this->browserInfo)) {
			$filename .= '_' . $this->browserInfo['browser'] . '-' . $this->browserInfo['version'] . '-' .  $this->browserInfo['os'];			
		} else {
			$filename .= '_generic';
		}

		// return name
		return $filename;
	}
	
	function getCacheFileNameImageArray(){
		$filename = $this->conf['caching']['cacheDirectory'];
		
		// type
		$filename .= 'imagearray';
		
		// set
		$filename .= '_' . $this->set;
		
		// browser
		if(is_array($this->browserInfo)) {
			$filename .= '_' . $this->browserInfo['browser'] . '-' . $this->browserInfo['version'] . '-' .  $this->browserInfo['os'];			
		} else {
			$filename .= '_generic';
		}

		// return name
		return $filename;
	}	
	
	function setHTTPHeaderFiletype(){
		header($this->conf['output']['fileheader_' . $this->type]);
	}


	function setHTTPHeaderCaching($filedate) {
		// just send 304 not modified if browser has file already in cache
		if(@strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']) == $filedate) {
		    header("HTTP/1.1 304 Not Modified");
			exit();
		}
		
		// set last modified and expires dates
		header("Last-Modified: ".gmdate("D, d M Y H:i:s", $filedate)." GMT");			
		$expires = $this->conf['caching']['cacheExpires'] * 24 * 60 * 60;
		header("Cache-Control: public, max-age=" . $this->conf['caching']['cacheExpires'] . $this->conf['caching']['cacheParams']);		
		header('Expires: ' . gmdate('D, d M Y H:i:s', time()+$this->conf['caching']['cacheExpires']) . ' GMT');
	}
	
	
	function setGzipCompression(){
		if($this->conf['output']['gzip']) {				
			// do not gzip for ie6 and ie7, because there is a bug in ie which disables the cache
			if($this->browserInfo['browser'] == "ie" && intval($this->browserInfo['major_version']) <= 7) {
				ob_start();
			} else {
				ob_start();
				ob_start("ob_gzhandler");
			}
		} else {
			ob_start();
		}
	}


/* ------------------------------------------------------------------------------------------------------------
   setters and getters
------------------------------------------------------------------------------------------------------------ */

	function setBrowserInformation($browserInfo) {
		
		// rewrite values is override for browser detection is enabled
		if($this->conf['browserDetection']['overrideDetection']) {
			if($this->conf['browserDetection']['overrideBrowser']) {
				$browserInfo['browser'] = $this->conf['browserDetection']['overrideBrowser'];
			}

			if($this->conf['browserDetection']['overrideOS']) {
				$browserInfo['os'] = $this->conf['browserDetection']['overrideOS'];
			}
			
			if($this->conf['browserDetection']['overrideEngine']) {
				$browserInfo['engine'] = $this->conf['browserDetection']['overrideEngine'];
			}
			
			if($this->conf['browserDetection']['overrideVersion']) {
				$browserInfo['version'] = $this->conf['browserDetection']['overrideVersion'];
			}
			
			if($this->conf['browserDetection']['overrideMajorVersion']) {
				$browserInfo['major_version'] = $this->conf['browserDetection']['overrideMajorVersion'];
			}
		} 
		
		// set values
		$this->browserInfo = $browserInfo;
	}

	function getConfig($section) {
		if($section) {
			return $this->conf[$section];
		} else {
			return $this->conf;
		}		
	}
	
	function getBaseDir(){
		return $this->baseDir;
	}

	
} // class phplivepacker end

?>